'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getCompilerState = getCompilerState;
exports.getBabel = getBabel;
exports.getOptions = getOptions;
exports.createCompilerState = createCompilerState;
exports.preCompileInitialization = preCompileInitialization;
exports.postprocess = postprocess;

var _babelTypesPatch = require('../util/babelTypesPatch');

var _babelTypesPatch2 = _interopRequireDefault(_babelTypesPatch);

var _lscNodeTypes = require('../lscNodeTypes');

var _stdlib = require('./stdlib');

var _runtime = require('./runtime');

var _imports = require('./imports');

var _helpers = require('./helpers');

var _inlinedOperators = require('../transforms/inlinedOperators');

var _astLocUtils = require('ast-loc-utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Singleton object, unique per Program traversal, holding relevant state.
var compilerState = {
  // DI Babel instance. Avoid unless necessary; prefer types.lsc
  babel: null

  // Root program path
  , programPath: null

  // Location mixin representing the first character in the file.
  , firstLoc: null

  // Babel Plugin options
  , opts: null

  // Babel file state entry
  , file: null

  //// Preambles
  // stdlib state
  , stdlib: null
  // helpers state
  , helpers: null
  // lightscript-runtime state
  , runtime: null,

  inlinedOperatorsEnabled: null,
  isLightScript: null,
  isLSX: null
}; // XXX

function getCompilerState() {
  return compilerState;
}function getBabel() {
  return compilerState.babel;
}function getOptions() {
  return compilerState.opts;
} // This will get the position of the first non-comment token. We place imports
// and requires here because ESLint requires the corresponding string literals
// to be on top of a valid token, or it chokes.
function findFirstTokenLoc(tokens) {
  if (!tokens || tokens.length === 0) return null;

  for (var _i2 = 0, _len2 = tokens.length; _i2 < _len2; _i2++) {
    var _t$type;

    var _t = tokens[_i2];
    if (_t && (!((_t$type = _t.type) == null ? void 0 : _t$type.indexOf) || _t.type.indexOf("Comment") < 0)) return (0, _astLocUtils.getLoc)(_t);
  }return null;
}function createCompilerState(babel) {
  // TODO: Eliminate global here. Compiler should pass state down using
  // `~` where necessary.
  compilerState = { babel };
  return compilerState;
}function preCompileInitialization(compiler, programPath) {
  var _file$ast, _programPath$node;

  var babel = compiler.babel,
      opts = compiler.opts,
      file = compiler.file;

  compiler.programPath = programPath;

  (0, _babelTypesPatch2.default)(babel.types);
  (0, _lscNodeTypes.registerLightscriptNodeTypes)(babel.types);

  // Get zero-location
  var tokens = file == null ? void 0 : (_file$ast = file.ast) == null ? void 0 : _file$ast.tokens;
  var body = programPath == null ? void 0 : (_programPath$node = programPath.node) == null ? void 0 : _programPath$node.body;
  var loc = findFirstTokenLoc(tokens);
  if (loc) {
    compiler.firstLoc = loc;
  } else if (body && body.length > 0) {
    compiler.firstLoc = (0, _astLocUtils.span)((0, _astLocUtils.getLoc)(body[0]), 1);
  } else {
    compiler.firstLoc = {
      start: 0,
      end: 1,
      loc: { start: { line: 1, column: 0 }, end: { line: 1, column: 1 } }
    };
  } // XXX: fix this
  compilerState.stdlib = (0, _stdlib.initializeStdlib)(opts);
  compilerState.inlinedOperatorsEnabled = (0, _inlinedOperators.getInlinedOperatorsEnabled)(opts);
  compilerState.runtime = (0, _runtime.initializeRuntime)(opts);
  (0, _helpers.resetHelpers)(programPath);

  return compilerState;
}function postprocess(state) {
  var stdlib = state.stdlib,
      programPath = state.programPath,
      runtime = state.runtime;

  (0, _imports.insertImports)(stdlib.imports, programPath, stdlib.useRequire || state.opts.useRequire);
  return (0, _imports.insertImports)(runtime.imports, programPath, state.opts.useRequire);
}